%% Creates datum record in global variable 'recline'
%% The datum contains matrices of equal numbers of rows:
%%
%%	recline.ch - channel (station) numbers consistent with those in
%%			GeoTomo files
%%	recline.x - in-line coodinates of the datum points. 
%%			This coordinate is set equal 0 at trace id (channel) = chan0
%%	recline.xyz - X,Y,Z coordinates of these points
%%
%% This function also executes line_distance(rec) and returns its outputs
%% ---------------------------------------------------------------------
%% Parameters:
%%      rec - Matrix of of receiver numbers and coordinates of points
%%	chan0 - Value of trace ID (channel, station number) at which 
%%		the inline distance is set equal 0

%%  Returns (same as line_distance()):
%%	in_line - column of in-line coordinates for 'rec'
%%      cross_line - cross-line offsets of points rec 
%%		from the nearest point on line 'xy'
%%      above_datum - hights of points st_xy above datum
%%	pxyz - matrix of X,Y,Z coordinates of the points on the datum line nearest to 'rec' 

function [ in_line, cross_line, above_datum, pxyz ] = datum( rec, chan0 )
 
global recline

  % create a smooth line along the recline

  recline.ch = transpose(linspace(-10,max(rec(:,1))+10,200));  % reference channel location grid

  recline.px = polyfit(rec(:,1),rec(:,2),3);	% polynomial function for x(distance)
  recline.py = polyfit(rec(:,1),rec(:,3),3);	% polynomial function for y(distance)
  recline.pz = polyfit(rec(:,1),rec(:,4),1);	% polynomial function for z(distance)

  recline.xyz = [ polyval(recline.px,recline.ch), polyval(recline.py,recline.ch), polyval(recline.pz,recline.ch) ];

  n = size(recline.xyz,1);

  % measure distance from the end of 'recline'

  recline.x = zeros(n,1);

  for i=2:n
	i1 = i-1;

	recline.x(i) = recline.x(i1) + distance(recline.xyz(i1,1:2),recline.xyz(i,1:2));
  end

  % set origin of in-line x so that it equals zero at recline.ch=chan0

  recline.x = recline.x - interp1(recline.ch,recline.x,chan0,'linear','extrap');

  % evaluate line_distance() for 'rec'

  [ in_line, cross_line, above_datum, pxyz ] = line_distance(rec(:,2:4));

  % shift datum z so that it is at the lowest receiver elevation

  z0 = min(above_datum);	% lowest point of rec relative to datum

  recline.xyz(:,3) = recline.xyz(:,3) + z0;	% lower the datum (z0 should be < 0)
  pxyz(:,3) = pxyz(:,3) + z0;
  above_datum = above_datum - z0;
 
end

