%% Inversion for delay times
%% Inverts for a delay times 'delta' associated with X coordinates in array 'x'
%% The inverted equation is:
%%	t_int = delta(x1) + delta(x2) 		(**)
%% 	
%%	where t_int is the input "intercept" time, and points x1 and x2 have the meanings of
%%	some source and receiver locations or of the points at which critical reflections occur.   
%%
%% Inversion is performed using the ordinary least squares algorithm.
%% 
%% For points in 'x' not covered by the data in 'coeffs', values of delta are interpolated
%%
%% Parameters:
%%      coeffs - input data matrix, with rows [ x1, x2, t_int ] in eq. (**)
%%	x - array of locations X at which the delays are evaluated. Between these coordinates,
%%		linear interpolation of 'delta' values is assumed.
%%  Returns:
%%	delta - column of values of delay-time terms in eq. (**) evaluated at points 'x'

function delta = invert_delay( coeffs, x )

	nx = length(x);

	delta = zeros(length(x),1);	% init output
 
	[ xg, ix ] = sort(x);

	nd = size(coeffs,1);	% number of data

	L = zeros(nd,nx);

	for i=1:nd

		x1 = coeffs(i,1);
		x2 = coeffs(i,2);

		ind1 = find(xg<x1);

		n = length(ind1);

		if n == 0 || n == nx	
			continue
		end

		ind2 = find(xg<x2);

		n = length(ind2);

		if n == 0 || n == nx	
			continue
		end

		ind1 = ind1(end);	% indices of points below x1 or x2
		ind2 = ind2(end);

		w = ( x1 - xg(ind1) ) / ( xg(ind1+1) - xg(ind1) );	% weight of the point xg(ind1+1)
		
		L(i,ind1) = 1-w;
		L(i,ind1+1) = w;

		w = ( x2 - xg(ind2) ) / ( xg(ind2+1) - xg(ind2) );	% weight of the point xg(ind2+1)
		
		L(i,ind2) = 1-w;
		L(i,ind2+1) = w;
	end

	% drop columns with small hit counts

	keep = sum(L);

	ind = find(keep>1.5);
	L = L(:,ind);

	% least squares

	Lt = transpose(L);

	d = pinv(Lt*L)*(Lt*coeffs(:,3));

	% interpolate using the successful values

	if length(ind) == nx		% all points found
		delta = d;
	else				% interpolate the covered region

		if ind(1)>1
			delta(1:(ind(1)-1),1) = d(1);
		end

		if ind(end)<nx
			delta((ind(end)+1):end,1) = d(end);
		end

		ir = ind(1):ind(end);

		delta(ir,1) = interp1(xg(ind),d,xg(ir),'linear');
	end

% test plot:
			
%figure(778)
%clf
%subplot(2,1,1)
%hold on
%plot(coeffs(:,3),'r+')
%plot(interp1(xg,delta,coeffs(:,1))+interp1(xg,delta,coeffs(:,2)),'kx')
%xlabel("Number")

%subplot(2,1,2)
%hold on
%plot(xg,delta,'m-')
%plot(xg(ind),d,'go')

%xlabel("x")

end

