%% Inversion for a layered model of the subsurface
%%
%% The inversion is performed in two steps:
%%
%% Step 1): For each branch (identified by the offset at the midpoint location),
%%	the measured travel times frim 'ttdata' are corrected for refractor moveout 
%%	by evaluating the reduced time:
%%
%% 		t_red = time - p*offset,
%%
%%	where 'p' os the slowness at the refractor in matrix 'model.p'.
%%	The reduced times are inverted for delay times 'delta' at the points
%%	of critical incidence onto the refractorfrom the source or receiver:
%%
%%		t_red = delta(x_source) + delta(x_receiver).
%% 	
%% Step 2): at each head-wave branch and profile coordinate x, the inverted  
%%	delay times delta(x) are transformed into refractor depths 
%%
%% Output refractor delays are placed in:
%%	matrix 'model.delta' (rows correspond to x coordinates in recline.midpoint 
%%			and columns to and columns to the different refractors)
%%	matrix 'model.z' (values of depths in the same format)
%% 
%% Parameters:
%%      none
%%  Returns:
%%	none

function invert_model()

global	recline src rec ttdata model procdata

	np = size(model.p,2);

	model.delta = zeros(size(model.p));	% init output matrix

	% determine coordinates and midpoints for each data point

	xsrc = src(ttdata(:,1),5);
	xrec = rec(ttdata(:,2),5);
	xmp = (xsrc + xrec)*0.5;

	% invert for each refractor independently

	offsmin = crossover( 1, xmp );	% min offsets for the first branch 

	for i=1:np
		offsmax = crossover( i+1, xmp );	% max offsets for i-th branch 

		% select all data for i-th branch

		aoffs = abs(ttdata(:,2));		% abs values of offsets
		soffs = sign(ttdata(:,2));		% signs of offsets

		ind = find(aoffs >= offsmin & aoffs >= offsmax);

		if ~isempty(ind)

			% ray parameters near source and receiver

			p_src = interp1(recline.midpoint_x,model.p(:,i),xsrc(ind,1),'linear','extrap');
			p_rec = interp1(recline.midpoint_x,model.p(:,i),xrec(ind,1),'linear','extrap');	

			d = soffs(ind,1);		% directions

			coeffs = [ x_critical(xsrc(ind,1),d,i), x_critical(xrec(ind,1),-d,i), ...
				   ttdata(ind,4) - 0.5*(p_src+p_rec).*aoffs(ind,1) ];
		end

		d = invert_delay(coeffs,model.inv1_x);
%coeffs
%d
		model.delta(:,i) = interp1(model.inv1_x,d,recline.midpoint);

		offsmin = offsmax;
	end

	% estimate depths of boundaries from intercept times and p values in the model

	model.z = refr_depth(model.delta,1);		% uppermost layer

	for i=2:size(model.p,2)
		model.z(:,i) = refr_depth(model.delta,i); 	
	end

end

