% Loads pick file from GeoTomo into global variables
% In the file, the line containing the names of columns and 
% any other text lines should be commented out using '%'. 
%
% The columns in output global matrices are as follows:
%
%	src - FFID number, X, Y, Z, inline, crossline, Z above datum, (X,Y,Z) on the line
%	rec - CHAN number, X, Y, Z, inline, crossline, Z above datum, (X,Y,Z) on line
%	ttdata - matrix with columns: (row number in rec, signed offset, first-break time)

%	(NOT USED) ttbranch - row number in src, row number in rec, first row in ttdata, last row in ttdata

% =============================================================================
% Parameters
%	filename - name of the file with all picks

function load_picks( filename )

global procdata ttdata ttbranch src rec recline 

	d = load(filename);

	src = d(:,[ 1, 3:5 ]);	% source columns

	rec = d(:,[ 2, 7:9 ]);	% receiver columns

	% extract unique source and receiver records sorted in ascending order

	[ ~, ind ] = unique(src(:,1));
	src = src(ind,:);

	[ ~, ind ] = unique(rec(:,1));
	rec = rec(ind,:);

	% create datum by approximating the receiver line
	% x = 0 is placed at chan = 1

	% calculate in-line positions of sources and receivers on the line

   	[ rec(:,5), rec(:,6), rec(:,7), rec(:,8:10) ] = datum(rec,1);
   	[ src(:,5), src(:,6), src(:,7), src(:,8:10) ] = line_distance( src(:,2:4) );

	% the above projection did not work well for these data. 
	% Looks like GPS coordinates are poor, X values vary by +/-5 m
	% Therefore, replacing in-line coordinates by fixed 1-m spacings of receivers
	% and source coordinates estimated from distances in data

   	[ rec(:,5), rec(:,6), rec(:,7), rec(:,8:10) ] = line_distance_chan( rec(:,2:4), rec(:,1)-1.0 );

	% place sources so that offsets match the data

   	[ src(:,5), src(:,6), src(:,7), src(:,8:10) ] = line_src(d(:,[ 1, 3:4 ]),d(:,[ 2, 7:8 ]));

	% fill ttdata

	[ ~, indsrc ] = ismember(d(:,1),src(:,1)); 	% row numbers in src for each data point
	[ ~, indrec ] = ismember(d(:,2),rec(:,1)); 	% row numbers in rec for each data point

	ttdata = [ indsrc, indrec, rec(indrec,5) - src(indsrc,5), d(:,11)*1000.0 ]; 
					% all offsets in meters and first break times in ms

	% apply time pick corrections to FFIDs specified in procdata.correct_tt

	for i=1:size(procdata.correct_tt,1)
		
		ind = find(d(:,1) == procdata.correct_tt(i,1));	  

		if ~isempty(ind)
			ttdata(ind,4) = ttdata(ind,4) + procdata.correct_tt(i,2);
		end
	end

	% apply source coordinate corrections to FFIDs specified in procdata.correct_src_x

	for i=1:size(procdata.correct_src_x,1)
		
		ind = find(d(:,1) == procdata.correct_src_x(i,1));	  

		if ~isempty(ind)
			srcno = ttdata(ind(1),1);	% our source number

			src(srcno,5) = src(srcno,5) + procdata.correct_src_x(i,2);	% shift source

			ttdata(ind,3) = ttdata(ind,3) - procdata.correct_src_x(i,2);
					% change offsets so that receivers remain at the same X
		end
	end
end

