%% Measures the direct-wave slowness model.p0, delay time model.delta0,
%% and layer slownesses model.p and intercepts model.tint at each midpoint
%% in recline.midpoint
%%
%% The estimates are obtained by inverting the average of the common-midpoint and common-offset
%% travel times as a linear regression:
%%
%%	time(offset) = delta0 + p0*offset
%%
%% The inverted parameters are returned and saved as:
%%	 model.p0 and model.delta0	(for direct wave)
%%	 model.p and model.tint		(for refractions)
%%
%% Additionally, for each row in 'ttdata', predicted linear-moveout part of the first-arrival times is
%% calculated and stored in global structure 'ttpred':
%%	ttpred.branch is a column of first-arrival branch numbers for each source-receiver pair in ttdata
%%			(0 for direct wave, >= 1 for refractions),
%%	ttpred.taup(:,1) is the first-arrival intercept time for this source-receiver pair,
%%	ttpred.taup(:,2) is the ray parameter for this source-receiver pair.
%% =========================================================================================
%%
%% Parameters:
%%      none
%%  Returns:
%%	p0 - direct-wave slowness (measured before the first crossover), sampled at every midpoint
%%	delta0 - near-surface delay (delay of the direct wave) sampled at every midpoint
%%	p - columns of this matrix are slownesses for all boundaries listed in model.crossover 
%%	intercept - columns of this matrix are intercepts for all boundaries listed in model.crossover 

function [ p0, delta0, p, intercept ] = measure_p_int()

global src rec recline ttdata ttpred model

	[ ttft, ttfb, offset ] = ttf( 0.25, recline.max_offset );	
	%[ ttft, ttfb, offset ] = ttf( 0.25, max(model.crossover(:,2))); % )recline.max_offset );	
				% offset grid at 0.25-m increments to max last crossover

	xmp = recline.midpoint_x;

	nx = length(xmp);
	noffs = length(offset);

	p0 = zeros(nx,1);		% init
	int0 = p0;
	delta0 = p0;

	p = zeros(nx,size(model.crossover,2)-1);	% one column for each crossover pick

	coeffs = [];		% this matrix will accumulate data for inversion

	for i=1:nx
		x0 = xmp(i);

		xt = ttimes_cmp(x0);	% common midpoint curve

		if isempty(xt)
			continue
		end

		for j=0:(size(model.crossover,2)-1)	% loop over all branches

			% select branch

			ind1 = find(xt(:,1)==j);
			ind2 = find(ttfb(i,:)==j);

			x2 = abs(offset(ind2));
			t2 = ttft(i,ind2);

%ttb = ttfb(i,:)
%x2
%b = [ x2; ttfb(i,ind2) ]

			% interpolate cmp xt into offset grid (for equal weights in polyfit)

			if length(ind1) >= 2
				x1 = x2;

				t1 = interp1(abs(xt(ind1,3)),xt(ind1,4),x1);

				ind = isfinite(t1);
				x1 = x1(ind);
				t1 = t1(ind);
			else
				x1 = abs(xt(ind1,3));
				t1 = xt(ind1,4);
			end

			% combine points from branch j into one regression

			[ int1, p1 ] = tau_p( [ x1, x2 ], [ t1, t2 ] );

			if ~isempty(p1)			% measured successfully

			   if j==0			% direct wave
				int0(i,1) = int1;
				p0(i,1) = p1;

				% add time misfit and source-receiver coordinates into 'coeffs'

				if ~isempty(ind1)
					offs = xt(ind1,3);

					coeffs = [ coeffs; x0 - offs*0.5, x0 + offs*0.5, (xt(ind1,4) - abs(offs)*p1) ];
				end
			   else					
		%		intercept(i,j) = int1;
				p(i,j) = p1;
			   end
			end

% test plot:
			
%if j == 0
%figure(777)
%hold on
%plot(abs(x1),t1,'r-')
%plot(abs(x2),t2,'b-')
%x = [ x1, x2 ];

%if ~isempty(p1)
%plot(x,x*p1+int1,'k-')
%end
%end

		end	% for j

	end	% for i

	% for deeper layers, filter the intercepts and slownesses spatially
 
	p = filter_spatial(p,model.smooth);
%	intercept = filter_spatial(intercept,model.smooth);

	% make sure slowness is not greater than in the preceding branch

	for i=1:nx
	
%		int_pred = int0(i,1);
		p_pred = p0(i,1);
		
		for j=1:size(p,2)
	
			if p(i,j) > p_pred
				p(i,j) = p_pred;
%				intercept(i,j) = int_pred;
			end

%			int_pred = intercept(i,j);
			p_pred = p(i,j);
		end
	end

%coeffs

	% invert direct-wave intercepts up to crossover distances for delays at midpoints

	d0 = invert_delay(coeffs,model.inv1_x);		% use denser grid for inversion

	delta0 = interp1(model.inv1_x,d0,xmp,'linear','extrap');

	% determine intercepts from crossover distances 

	intercept = ttimes_intercept(p0,p,xmp);

	% save results in 'model'

	model.p0 = p0;
	model.delta0 = delta0;

	model.p = p;
	model.tint = intercept;

	% estimate depths of boundaries from intercept times and p values in the model

	del = 0.5*model.tint;		% delays estimated as half intercepts for now

	model.z = refr_depth(del,1);		% uppermost layer

	for i=2:size(model.p,2)
		model.z(:,i) = refr_depth(del,i); 	
	end

	% save predicted travel times in 'ttpred.taup'

	offs = abs(ttdata(:,3));				% source-receiver offsets
	xmp = (src(ttdata(:,1),5) + rec(ttdata(:,2),5))*0.5;	% midpoint coordinates

        [ ~, cmpnum ] = midpoints(xmp);		% midpoint numbers

	p1 = [ p0(cmpnum,1), p(cmpnum,:) ];			% ray parameters for all branches
	int1 = [ delta0(cmpnum,1), intercept(cmpnum,:) ];	% intercepts for all branches

	t1 = offs.*p1 + int1;			% direct-wave and all head-wave times
	
	[ t2, ind ] = min(t1,[],2);		% first-arrival times (smallest among the branches)

	ttpred.branch = ind - 1;		% branch numbers
	
	ttpred.taup = zeros(size(ttdata,1),2);

	for i=1:size(ind,1)
		ttpred.taup(i,:) = [ int1(i,ind(i)), p1(i,ind(i)) ];
	end

end	

