%% Extracts travel-time points for a given source within one or several offset ranges
%%
%% Output travel-time branches are in matrices with columns:
%%		1) branch number (0 for direct wave, 1 for refraction, etc.)
%%			determined by comparison with crossover distance table model.crossover
%%		2) receiver number
%%		3) offset
%%		4) time
%% ------------------------------------------------------------------------------------
%% Parameters:
%%      srcno - source number (row in matrix src)
%%	offset_range - if not empty, this array of two values gives the range of offsets
%%		(second column in 'xt') to output. 
%%		The values in the row can be given in arbitrary order.
%%		If multiple rows are given, multiple travel-time branches 
%%		are concatenated 'xt'
%%	rec_x -  receiver positions for interpolating travel times
%%		If this array is not given or is empty, the original receiver positions are output
%%  Returns:
%%	xt - matrix with columns (branch_number, receiver_number, offset, time)
%% ========================================================================================

function xt = ttimes_src( srcno, offset_range, rec_x )
 
global src rec ttdata model procdata

	if nargin < 2
		offset_range = [];
	else
	   	offset_range = sort(offset_range,2);
	end

	if nargin < 3
		rec_x = [];
	end

	xt = [];

	ind = find(ttdata(:,1)==srcno);

	if isempty(ind)
		return
	end
		
	defbranch = size(model.crossover,2);	% default branch number (largest possible)

	xt1 = ttdata(ind,2:4);

	% subtract corrections if requested

	switch tolower(procdata.correct_ttimes)

		case 'delta0'			% subtract zero-offset delays

			ttp = ttimes_predict(0);

			xt1(:,3) = xt1(:,3) - ttp(ind,1);
	end

	% select offset ranges

	if isempty(offset_range)
		xt = [ ones(size(xt1,1),1)*defbranch, xt1 ];
	else
	  for i=1:size(offset_range,1)

		ind = find(xt1(:,2) >= offset_range(i,1) & xt1(:,2) < offset_range(i,2) );

		if ~isempty(ind)
			xt = [ xt; ones(size(ind,1),1)*defbranch, xt1(ind,:) ];
		end
	  end
	end

	% interpolate values at requested points

	if ~isempty(rec_x) 
		xt1 = xt;
		xt = [];

		offset = rec_x - src(srcno,5);

		% interpolate reverse branches 

		offs1 = offset(find(offset<= 0.0));

		ind1 = find(xt1(:,3)<=0.0);

		if size(ind1,1) >= 2 
			t = interp1(xt1(ind1,3),xt1(ind1,4), offs1);
			r = interp1(xt1(ind1,3),xt1(ind1,2), offs1);

			% output only correctly interpolated values

			ind = find(isfinite(t));

			if ~isempty(ind)
				xt = [ xt; ones(size(ind,1),1)*defbranch, r(ind,:), offset(ind,:), t(ind,:) ];
			end
		end

		% interpolate forward branches 

		offs1 = offset(find(offset>= 0.0));

		ind1 = find(xt1(:,3)>=0.0);

		if size(ind1,1) >= 2 
			t = interp1(xt1(ind1,3),xt1(ind1,4), offs1);
			r = interp1(xt1(ind1,3),xt1(ind1,2), offs1);

			% output only correctly interpolated values

			ind = find(isfinite(t));

			if ~isempty(ind)
				xt = [ xt; ones(size(ind,1),1)*defbranch, r(ind,:), offset(ind,:), t(ind,:) ];
			end
		end
	end

	% set branch numbers below defbranch

	xt = set_branch( xt, src(srcno,5) );

end

