%% Returns horizontal position of the point of critical incidence on a refractor 
%% ------------------------------------------------------------------------------------
%% Parameters:
%%      x - coordinate of the source or receiver on the surface 
%%      direction - vector of the same size as 'x' indicating directions in which the critical
%%		points are sought: for direction = +1, xcrit > x is returned, 
%%					for direction = -1, xcrit < x
%%	 
%%	irefr - refractor number (>= 1). If irefr < 1 or refractor does not exist,
%%		the function returns the value of 'x'
%%  Returns:
%%	xcrit - horizontal distance from 'x' to the point of critical-angle incidence
%%		on the refractor
%% ========================================================================================

function xcrit = x_critical( x, direction, irefr )
	
global	recline model

	if irefr < 1 || irefr > size(model.p,2)		% no such refractor
		xcrit = x;
		return
	end

	prefr = interp1(recline.midpoint,model.p(:,irefr),x,'linear','extrap');	% p of refractor

	z = interp1(recline.midpoint,model.z(:,1),x,'linear','extrap');	% depth of first boundary
	p = interp1(recline.midpoint,model.p0,x,'linear','extrap'); 	% p of first boundary
	theta = real(asin(prefr./p)); 					% angle above boundary 1

	xcrit = z.*tan(theta);

	for i=2:irefr
		z_pred = z;

		z = interp1(recline.midpoint,model.z(:,i),x,'linear','extrap');	% depth of boundary i
		p = interp1(recline.midpoint,model.p(:,i),x,'linear','extrap'); % p of boundary i

		theta = real(asin(prefr./p)); 		% angle above boundary i

		xcrit = xcrit + (max([z,z_pred],[],2) - z_pred).*tan(theta);
	end

%sd = size(direction)
%sx = size(x)

	xcrit = x + xcrit.*direction;
end

